% CAPI vs. CAWI, 2020-03-22


%% Prepare and clean data:

clear all; clc;
DATA = load('data_all_countries_02.mat');

CTR = {'Germany','Denmark','Estonia','Finland','Latvia','Lithuania','Poland_CAPI','Poland_CAWI','Russia','Sweden'};

DATA.CTR = cell(size(DATA.CountryId));
DATA.CTR(DATA.CountryId==4) = {'Germany'};
DATA.CTR(DATA.CountryId==1) = {'Denmark'};
DATA.CTR(DATA.CountryId==2) = {'Estonia'};
DATA.CTR(DATA.CountryId==3) = {'Finland'};
DATA.CTR(DATA.CountryId==5) = {'Latvia'};
DATA.CTR(DATA.CountryId==6) = {'Lithuania'};
DATA.CTR(DATA.CountryId==7 & DATA.Method==2) = {'Poland_CAPI'}; % to trzeba sprawdzi?, czy nie jest odwrotnie
DATA.CTR(DATA.CountryId==7 & DATA.Method==1) = {'Poland_CAWI'};
DATA.CTR(DATA.CountryId==8) = {'Russia'};
DATA.CTR(DATA.CountryId==9) = {'Sweden'};

DATA.CAPI = zeros(size(DATA.CTR));
DATA.CAPI(strcmp(DATA.CTR,['Latvia'])) = 1;
DATA.CAPI(strcmp(DATA.CTR,['Lithuania'])) = 1;
DATA.CAPI(strcmp(DATA.CTR,['Poland_CAPI'])) = 1;
DATA.CAPI(strcmp(DATA.CTR,['Russia'])) = 1;

% DATA.bounds = [DATA.WTPfpL, DATA.WTPfpU]; % trzeba sprawdzi?, czy dla polski to jest fp czy hp... 
% DATA.bounds(DATA.Q21==0,:) = 0;
% DATA.bounds(DATA.bounds(:,2) >= 1000000,2) = Inf;

DATA.Q26(DATA.Q26==19) = 18; % 19 is 'more than the highest bid' - recoded to the highest bid
DATA.Q23(DATA.Q23==19) = 18;

% recoding bounds (the ones in the data are not always consistent with responses to Q26)
bids = [4,0,5,7,9,12,15,20,28,37,50,65,90,115,150,200,270,360,480,Inf,NaN,NaN;1,0,45,60,80,110,145,190,250,340,450,600,800,1100,1425,1900,2500,3375,4500,Inf,NaN,NaN;2,0,2,3,4,5,6,8,10,15,20,25,35,50,65,80,115,150,200,Inf,NaN,NaN;3,0,5,7,10,15,20,25,35,50,75,100,140,200,300,400,550,750,1000,Inf,NaN,NaN;5,0,0.200000000000000,0.300000000000000,0.500000000000000,0.700000000000000,1,1.50000000000000,2,3,4,5,10,12,20,30,40,60,80,Inf,NaN,NaN;6,0,5,7,9,11,15,20,25,35,45,60,80,100,130,180,230,300,400,Inf,NaN,NaN;7,0,0.0500000000000000,7,10,15,20,25,30,40,50,60,75,100,125,175,225,300,400,Inf,NaN,NaN;7,0,0.0500000000000000,7,10,15,20,25,30,40,50,60,75,100,125,175,225,300,400,Inf,NaN,NaN;8,0,30,45,60,80,400,550,750,1000,1500,2000,3000,4000,6000,0,0,0,0,Inf,NaN,NaN;9,0,70,100,150,200,300,400,500,750,1000,1500,2000,3000,4000,6000,8000,11000,15000,Inf,NaN,NaN]; % the first is country ID
boundsF = NaN(size(DATA.CTR,1),2);
bounds_tmp = vlookup(DATA.CountryId,bids(:,1),bids(:,2:end));
Q26_idx = DATA.Q26;
Q26_idx(Q26_idx==99) = 20;
Q26_idx(isnan(Q26_idx)) = 20;
for i = 1:length(bounds_tmp)
    boundsF(i,1) = bounds_tmp(i,Q26_idx(i));
    boundsF(i,2) = bounds_tmp(i,Q26_idx(i)+1);
end
boundsF(DATA.Q21==0,:) = 0; % not in the market

boundsH = NaN(size(DATA.CTR,1),2);
bounds_tmp = vlookup(DATA.CountryId,bids(:,1),bids(:,2:end));
Q23_idx = DATA.Q23;
Q23_idx(Q23_idx==99) = 20;
Q23_idx(isnan(Q23_idx)) = 20;
for i = 1:length(bounds_tmp)
    boundsH(i,1) = bounds_tmp(i,Q23_idx(i));
    boundsH(i,2) = bounds_tmp(i,Q23_idx(i)+1);
end
boundsH(DATA.Q21==0,:) = 0; % not in the market

idx = boundsH(:,1) > boundsF(:,1); % WTP for H higher than for F
idx(~isnan(boundsH(:,1)) & boundsH(:,1)~=0 & isnan(boundsF(:,1))) = 1; % missing F while positive H

DATA.boundsF = boundsF;
DATA.boundsF(idx,:) = boundsH(idx,:);

clear boundsF boundsH bounds_tmp bids i idx Q23_idx Q26_idx

fields = fieldnames(DATA);

exchange_rates = [... % PPP-corrected exchange rates to EUR
1.090084868;... % Germany
10.56297517;... % Denmark
0.728655532;... % Estonia
1.261838731;... % Finland
0.493682794;... % Latvia
2.198372184;... % Lithuania
2.54055369;... % Poland_CAPI
2.54055369;... % Poland_CAWI
24.11267316;... % Russia
12.30730168]; % Sweden

for j = 1:length(CTR)
    
    for i = 1:length(fields)
        DATA2.(CTR{j}).(fields{i}) = DATA.(fields{i})(strcmp(DATA.CTR,CTR{j}),:);
    end
    
    DATA2.(CTR{j}).income = DATA2.(CTR{j}).INCOCU;
    DATA2.(CTR{j}).income(isnan(DATA2.(CTR{j}).income)) = nanmean(DATA2.(CTR{j}).income); % arbitrary imputation of mean income
    DATA2.(CTR{j}).boundsF(isinf(DATA2.(CTR{j}).boundsF(:,2)),2) = DATA2.(CTR{j}).income(isinf(DATA2.(CTR{j}).boundsF(:,2))); % replace upper bound = Inf with respondents's income
    DATA2.(CTR{j}).boundsF(DATA2.(CTR{j}).boundsF(:,2)<DATA2.(CTR{j}).boundsF(:,1),2) = DATA2.(CTR{j}).boundsF(DATA2.(CTR{j}).boundsF(:,2)<DATA2.(CTR{j}).boundsF(:,1),1); % if after income imputation higher bound lower than lower - make it equal to lower
    
    DATA2.(CTR{j}).boundsF = DATA2.(CTR{j}).boundsF ./ exchange_rates(j); % convert to EUR @ PPP
    DATA2.(CTR{j}).income = DATA2.(CTR{j}).income ./ exchange_rates(j); % convert to EUR @ PPP
   
end

DATA = DATA2;
clear DATA2 exchange_rates fields i j


%% import weights


WEIGHTS = load('data_all_countries_weights_02.mat');

for j = 1:length(CTR)
    DATA.(CTR{j}).weights(~any(isnan(DATA.(CTR{j}).boundsF),2)) = WEIGHTS.(CTR{j}).weights;
    DATA.(CTR{j}).weights = DATA.(CTR{j}).weights';    
end

clear WEIGHTS j 


%% Use country-specific data to fit parametric distributions:


% load('results_ewa.mat')
load('results ALL_new_3.mat')

INPUT.Censor0 = 1; % this treats WTP < 0 as 0

INPUT.MaxIter = 1e3;
INPUT.HessEstFix = 3; % available options: 0 - retained from optimization, 1 - BHHH based, 2 - numerical high-precision Jacobian based, 3 - numerical Hessian based
INPUT.SimStats = 1; % simulate descriptive statistics of the distribution

clear Results_summary

try

    for j = 1:length(CTR)
       
    INPUT.bounds = DATA.(CTR{j}).boundsF;
    INPUT.WT = DATA.(CTR{j}).weights; % weight observations

    INPUT.WT(any(isnan(INPUT.bounds),2),:) = [];
    INPUT.bounds(any(isnan(INPUT.bounds),2),:) = [];

    %% fit parametric distributions to data:

    INPUT.Spike = 0; % no zero inflation
    INPUT.Results = Results0.(CTR{j});
%     INPUT.Results = [];
    [Results, Results_summary0] = FitParametricDistributions_Baltic(INPUT);

    Results0.(CTR{j}) = Results; % Results without zero-inflation
  
    INPUT.Spike = 1; % with zero inflation
    INPUT.Results = Results1.(CTR{j});
%     INPUT.Results = [];
    [Results, Results_summary1] = FitParametricDistributions_Baltic(INPUT);    
    Results1.(CTR{j}) = Results; % Results with zero-inflation
       
    Results_summary0 = [Results_summary0(:,1:2),num2cell(zeros(size(Results_summary0(:,1)))),Results_summary0(:,3:end)];
    Results_summary1 = [Results_summary1(:,1:2),num2cell(ones(size(Results_summary1(:,1)))),Results_summary1(:,3:end)];
    Results_summary_j = [Results_summary0; Results_summary1];
    Results_summary_j = [{'Distribution','Log-likelihood','Zero inflation','AIC/n','BIC/n','n - observations','k - parameters','WTP (mean)','WTP (s.e.)','RealMin'};Results_summary_j];
    Results_summary.(CTR{j}) = Results_summary_j; % Save summary of results
    
    
    end

    save('results ALL_new_3')
    
    sendmail('mik@czaj.org','MATLAB: D5 - done!');
    
%     clear Results_summary0 Results_summary1 Results_summary_j Results j DATA INPUT   
  
catch theErrorInfo
    
    sendmail('mik@czaj.org','MATLAB: D5 - error! ',theErrorInfo.message);
    rethrow(theErrorInfo)

end


%% Lewbel-Watanabe non-parametric estimate


for j = 1:length(CTR)
    
%     DATA.(CTR{j}).WEIGHT = ones(size(DATA.(CTR{j}).CTR));
    
    % with weighting:
    bids = unique(DATA.(CTR{j}).boundsF(~isnan(DATA.(CTR{j}).boundsF(:,1)),1));
    Frequency_table = zeros(length(bids),7);
    Frequency_table(:,1) = bids;
    for i = 1:length(bids)
        Frequency_table(i,2) = sum(DATA.(CTR{j}).weights(DATA.(CTR{j}).boundsF(:,1) == bids(i))); % no of people who voted yes (weighted)
    end
    
    Frequency_table(:,3) = cumsum(Frequency_table(:,2),'reverse'); % implied # of 'yes'
    Frequency_table(:,7) = sum(Frequency_table(:,2)); % no of people total (weighted)
    Frequency_table(:,4) = Frequency_table(:,7) - Frequency_table(:,3); % implied # of 'no'
    
%     [WTPA, freq_tableA] = KaplanMeierEstimator(DATA.bid,DATA.Q27==1); % without weighting
%     Scale = freq_tableA(:,4)./Frequency_table(:,4);
%     Frequency_table(:,2:4) = Frequency_table(:,2:4).*Scale; % Scaling, so that the total number of individuals would not change

    Frequency_table(:,5) = Frequency_table(:,3) ./ Frequency_table(:,7); % implied % of 'yes'
    Frequency_table(:,6) = Frequency_table(:,4) ./ Frequency_table(:,7); % implied % of 'no'

    % LW (lower bound) estimate of WTP for A:
    WTP = sum((Frequency_table(:,1)-[0;Frequency_table(1:end-1,1)]).*Frequency_table(:,5)); % mean WTP
    SD = sqrt(sum(Frequency_table(:,5).*(Frequency_table(:,6))./Frequency_table(:,3).*((Frequency_table(:,1)-[0;Frequency_table(1:end-1,1)])).^2)); %. s.e. (based on Haab McConnell)

    Results_summary.(CTR{j}) = [Results_summary.(CTR{j})(1,:); ...
        {'Lewbel-Watanabe','','','','','','',WTP,SD,''};...
        Results_summary.(CTR{j})(2:end,:)];
end
clear i idx j 


%% extract parameters and standard errors for each model (to see if they converged to reasonable values)


for j = 1:length(CTR)
    Results_summary.(CTR{j})(1,[11,12,13,14,15]) = {'beta'};
    Results_summary.(CTR{j})(1,[16,17,18,19,20]) = {'std'};
    for i = 1:length(Results_summary.(CTR{j}))-2
        idx = Results_summary.(CTR{j}){i+2,3}; % spike indicator
        if idx == 0
            beta = Results0.(CTR{j}).(Results_summary.(CTR{j}){i+2,1}).beta';
            std = Results0.(CTR{j}).(Results_summary.(CTR{j}){i+2,1}).std';
        elseif idx == 1
            beta = Results1.(CTR{j}).(Results_summary.(CTR{j}){i+2,1}).beta';
            std = Results1.(CTR{j}).(Results_summary.(CTR{j}){i+2,1}).std';
        end
        Results_summary.(CTR{j})(i+2,11:11+size(beta,2)-1) = num2cell(beta);
        Results_summary.(CTR{j})(i+2,16:16+size(std,2)-1) = num2cell(std);
    end
end         


%% drop models that did not converge


for j = 1:length(CTR)
    clear idx
    for i = 3:size(Results_summary.(CTR{j}),1)
        idx(i,1) = (Results_summary.(CTR{j}){i,2} >= 0) | isnan(Results_summary.(CTR{j}){i,2}) | isinf(Results_summary.(CTR{j}){i,2}) | ... % criteria for LL        
        any(isnan([Results_summary.(CTR{j}){i,8:9}]),2) | any(isinf([Results_summary.(CTR{j}){i,8:9}]),2) | any([Results_summary.(CTR{j}){i,8:9}] > 1e3,2) | any([Results_summary.(CTR{j}){i,8}] < 1,2) | ... % conditions on WTP and its s.e.
        any(([Results_summary.(CTR{j}){i,9}]) > 0.5*([Results_summary.(CTR{j}){i,8}]),2) | ... % s.e. > 0.5 mean WTP
        any(isnan([Results_summary.(CTR{j}){i,10:19}]),2) | any(isinf([Results_summary.(CTR{j}){i,10:19}]),2) | any([Results_summary.(CTR{j}){i,10:19}] > 1e3,2); % | ... conditions on betas and stds
        % any([Results_summary.(CTR{j}){i,10}] == 1,2); % criteria for RealMin
    end
    Results_summary.(CTR{j})(idx==1,:) = [];
end
clear i idx j 


%% sort results according to fit


for j = 1:length(CTR)
    Results_summary.(CTR{j})(3:end,:) = sortrows(Results_summary.(CTR{j})(3:end,:),[5,4]);
end 


%% retain best starting values from two datasets:

R1 = load('results ALL_new_3.mat');
R2 = load('results ALL_new_3i.mat');

for j = 1:length(CTR)
    k = 0;
    for i=1:length(R1.Results0.(CTR{j}).Distributions)
        if ((R1.Results0.(CTR{j}).(R1.Results0.(CTR{j}).Distributions{i,2}).fval < R2.Results0.(CTR{j}).(R2.Results0.(CTR{j}).Distributions{i,2}).fval) && (R2.Results0.(CTR{j}).(R2.Results0.(CTR{j}).Distributions{i,2}).RealMin == 0)) || ...
                ((R2.Results0.(CTR{j}).(R2.Results0.(CTR{j}).Distributions{i,2}).RealMin == 0) && (R1.Results0.(CTR{j}).(R1.Results0.(CTR{j}).Distributions{i,2}).RealMin == 1))
            k = k+1;
            R1.Results0.(CTR{j}).(R1.Results0.(CTR{j}).Distributions{i,2}).beta = R2.Results0.(CTR{j}).(R2.Results0.(CTR{j}).Distributions{i,2}).beta;
            R1.Results0.(CTR{j}).(R1.Results0.(CTR{j}).Distributions{i,2}).fval = R2.Results0.(CTR{j}).(R2.Results0.(CTR{j}).Distributions{i,2}).fval;
            R1.Results0.(CTR{j}).(R1.Results0.(CTR{j}).Distributions{i,2}).RealMin = R2.Results0.(CTR{j}).(R2.Results0.(CTR{j}).Distributions{i,2}).RealMin;
        end
        if ((R1.Results1.(CTR{j}).(R1.Results1.(CTR{j}).Distributions{i,2}).fval < R2.Results1.(CTR{j}).(R2.Results1.(CTR{j}).Distributions{i,2}).fval) && (R2.Results1.(CTR{j}).(R2.Results1.(CTR{j}).Distributions{i,2}).RealMin == 0)) || ...
                ((R2.Results1.(CTR{j}).(R2.Results1.(CTR{j}).Distributions{i,2}).RealMin == 0) && (R1.Results1.(CTR{j}).(R1.Results1.(CTR{j}).Distributions{i,2}).RealMin == 1))
            k = k+1;
            R1.Results1.(CTR{j}).(R1.Results1.(CTR{j}).Distributions{i,2}).beta = R2.Results1.(CTR{j}).(R2.Results1.(CTR{j}).Distributions{i,2}).beta;
            R1.Results1.(CTR{j}).(R1.Results1.(CTR{j}).Distributions{i,2}).fval = R2.Results1.(CTR{j}).(R2.Results1.(CTR{j}).Distributions{i,2}).fval;
            R1.Results1.(CTR{j}).(R1.Results1.(CTR{j}).Distributions{i,2}).RealMin = R2.Results1.(CTR{j}).(R2.Results1.(CTR{j}).Distributions{i,2}).RealMin;
        end
    end
    k
end
% i = i+1;
save('results ALL_new_3.mat','-struct','R1') %- this is the newest results file


%% Plot CAPI vs. CAWI WTP functions:

j = 7; % Poland_CAPI, 

% INPUT.Spike = 0; % allow for jump density at 0
%     
% INPUT.HessEstFix = 3; % available options: 0 - retained from optimization, 1 - BHHH based, 2 - numerical high-precision Jacobian based, 3 - numerical Hessian based
% INPUT.SimStats = 1; % simulate descriptive statistics of the distribution
% INPUT.Censor0 = 1; % this treats WTP < 0 as 0.
% 
% INPUT.bounds = DATA.(CTR{j}).boundsF;
% INPUT.WT = DATA.(CTR{j}).weights; % weight observations
% INPUT.WT(any(isnan(INPUT.bounds),2),:) = [];
% INPUT.bounds(any(isnan(INPUT.bounds),2),:) = [];
% % Results_CAPIesults = DistFit(INPUT,13,Results1.Poland_CAPI.Weibull.beta(1:2));
% INPUT.Algorithm = 'search';
% INPUT.Algorithm = 'interior-point';
% INPUT.Algorithm = 'sqp';
% Results_CAPI = DistFit(INPUT,13,Betas_param);

    bids = unique(DATA.(CTR{j}).boundsF(~isnan(DATA.(CTR{j}).boundsF(:,1)),1));

    Frequency_table = zeros(length(bids),7);
    Frequency_table(:,1) = bids;
    for i = 1:length(bids)
        Frequency_table(i,2) = sum(DATA.(CTR{j}).weights(DATA.(CTR{j}).boundsF(:,1) == bids(i))); % no of people who voted yes (weighted)
    end    
    Frequency_table(:,3) = cumsum(Frequency_table(:,2),'reverse'); % implied # of 'yes'
    Frequency_table(:,7) = sum(Frequency_table(:,2)); % no of people total (weighted)
    Frequency_table(:,4) = Frequency_table(:,7) - Frequency_table(:,3); % implied # of 'no'
    Frequency_table(:,5) = Frequency_table(:,3) ./ Frequency_table(:,7); % implied % of 'yes'
    Frequency_table(:,6) = Frequency_table(:,4) ./ Frequency_table(:,7); % implied % of 'no'

    SEA = sqrt(Frequency_table(:,5).*Frequency_table(:,6)./Frequency_table(:,3)); % s.e. for weighted data (based on Haab McConnell)

figure(1)

xx = linspace(0,max(bids),500);

% Betas_param = Results1.(CTR{j}).Lognormal.beta';
% pSpike = normcdf(Betas_param(3)); % p1
% wblF = pSpike + (1-pSpike).*cdf('Lognormal',xx,Betas_param(1),Betas_param(2));
Betas_param = Results1.(CTR{j}).Generalized_Pareto.beta';
pSpike = normcdf(Betas_param(4)); % p1
wblF = pSpike + (1-pSpike).*cdf('Generalized Pareto',xx,Betas_param(1),Betas_param(2),Betas_param(3));

sim1 = 1e5;
Betas_param_sim = mvnrnd(Betas_param,Results1.(CTR{j}).Generalized_Pareto.ihess,sim1)';
wblF_sim = zeros(sim1,500);
parfor i = 1:sim1
    pSpike = normcdf(Betas_param_sim(4,i)); % p1
    wblF_sim(i,:) = pSpike + (1-pSpike).*cdf('Generalized Pareto',xx,Betas_param_sim(1,i),Betas_param_sim(2,i),Betas_param_sim(3,i));    
end

wblF_sim_l = quantile(wblF_sim,0.025);
wblF_sim_u = quantile(wblF_sim,0.975);
hold on;
KMplot(1) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)],'b-','color','#00e64d'); % Taking NO answers
KMplot(2) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)+norminv(0.025)*[0;SEA(2:end)]],'b--','color','#00e64d');
KMplot(3) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)+norminv(0.975)*[0;SEA(2:end)]],'b--','color','#00e64d');
DISTplot(1) = plot(xx,wblF,'r-','color','#006622');
DISTplot(2) = plot(xx,wblF_sim_l,'r--','color','#006622');
DISTplot(3) = plot(xx,wblF_sim_u,'r--','color','#006622');
set(KMplot(1),'linewidth',2);
set(DISTplot(1),'linewidth',2);
% hold off
xlabel('WTP'); ylabel('Probability'); title('Empirical CDF')
set(gca, 'YGrid', 'on', 'GridLineStyle', ':')
ylim([0,1])
% set(gca,'XTick', freq_tableA(:,1))
% set(gca,'XTick', Frequency_table(:,1))
set(gca,'Position',[.13 .11 .85 .82]);
set(gcf,'Color', 'w')


j = 8; % Poland_CAWI, 

    bids = unique(DATA.(CTR{j}).boundsF(~isnan(DATA.(CTR{j}).boundsF(:,1)),1));

    Frequency_table = zeros(length(bids),7);
    Frequency_table(:,1) = bids;
    for i = 1:length(bids)
        Frequency_table(i,2) = sum(DATA.(CTR{j}).weights(DATA.(CTR{j}).boundsF(:,1) == bids(i))); % no of people who voted yes (weighted)
    end    
    Frequency_table(:,3) = cumsum(Frequency_table(:,2),'reverse'); % implied # of 'yes'
    Frequency_table(:,7) = sum(Frequency_table(:,2)); % no of people total (weighted)
    Frequency_table(:,4) = Frequency_table(:,7) - Frequency_table(:,3); % implied # of 'no'
    Frequency_table(:,5) = Frequency_table(:,3) ./ Frequency_table(:,7); % implied % of 'yes'
    Frequency_table(:,6) = Frequency_table(:,4) ./ Frequency_table(:,7); % implied % of 'no'

    SEA = sqrt(Frequency_table(:,5).*Frequency_table(:,6)./Frequency_table(:,3)); % s.e. for weighted data (based on Haab McConnell)

% Betas_param = Results1.(CTR{j}).Lognormal.beta';
% pSpike = normcdf(Betas_param(3)); % p1
% wblF = pSpike + (1-pSpike).*cdf('Lognormal',xx,Betas_param(1),Betas_param(2));
Betas_param = Results1.(CTR{j}).Exponential.beta';
pSpike = normcdf(Betas_param(2)); % p1
wblF = pSpike + (1-pSpike).*cdf('Exponential',xx,Betas_param(1));

sim1 = 1e5;
Betas_param_sim = mvnrnd(Betas_param,Results1.(CTR{j}).Exponential.ihess,sim1)';
wblF_sim = zeros(sim1,500);
parfor i = 1:sim1
    pSpike = normcdf(Betas_param_sim(2,i)); % p1
    wblF_sim(i,:) = pSpike + (1-pSpike).*cdf('Exponential',xx,Betas_param_sim(1,i));    
end

wblF_sim_l = quantile(wblF_sim,0.025);
wblF_sim_u = quantile(wblF_sim,0.975);
hold on;
CAWI_KM_plot(1) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)],'c-','color','#9999ff'); % Taking NO answers
CAWI_KM_plot(2) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)+norminv(0.025)*[0;SEA(2:end)]],'c--','color','#9999ff');
CAWI_KM_plot(3) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)+norminv(0.975)*[0;SEA(2:end)]],'c--','color','#9999ff');
CAWI_DIST_plot(1) = plot(xx,wblF,'r-','color','#0000ff');
CAWI_DIST_plot(2) = plot(xx,wblF_sim_l,'--','color','#0000ff');
CAWI_DIST_plot(3) = plot(xx,wblF_sim_u,'r--','color','#0000ff');
set(CAWI_KM_plot(1),'linewidth',2);
set(CAWI_DIST_plot(1),'linewidth',2);
hold off
xlabel('WTP'); ylabel('Probability'); title('Empirical CDF')
set(gca, 'YGrid', 'on', 'GridLineStyle', ':')
ylim([0,1])
% set(gca,'XTick', freq_tableA(:,1))
% set(gca,'XTick', Frequency_table(:,1))
set(gca,'Position',[.13 .11 .85 .82]);
set(gcf,'Color', 'w')

% legend([KMplot(1),KMplot(2), DISTplot(1),DISTplot(2)],'ABERS (lower bound)','95% confidence interval','Fitted gamma distribution','95% confidence interval', 'Location', 'southeast')
legend([KMplot(1),KMplot(2), DISTplot(1),DISTplot(2),CAWI_KM_plot(1),CAWI_KM_plot(2),CAWI_DIST_plot(1),CAWI_DIST_plot(2)],'CAPI - Lewbel-Watanabe (lower bound)','95% confidence interval','CAPI - Fitted parametric distribution','95% confidence interval','CAWI - Lewbel-Watanabe (lower bound)','95% confidence interval','CAWI - Fitted parametric distribution','95% confidence interval', 'Location', 'southeast')

saveas(gcf,'CAPIvCAWI','png')


%% Plot Normal vs. best fitting distribution for each country

sim1 = 1e5;

for j = 1:length(CTR)

    bids = unique(DATA.(CTR{j}).boundsF(~isnan(DATA.(CTR{j}).boundsF(:,1)),1));

    Frequency_table = zeros(length(bids),7);
    Frequency_table(:,1) = bids;
    for i = 1:length(bids)
        Frequency_table(i,2) = sum(DATA.(CTR{j}).weights(DATA.(CTR{j}).boundsF(:,1) == bids(i))); % no of people who voted yes (weighted)
    end
    Frequency_table(:,3) = cumsum(Frequency_table(:,2),'reverse'); % implied # of 'yes'
    Frequency_table(:,7) = sum(Frequency_table(:,2)); % no of people total (weighted)
    Frequency_table(:,4) = Frequency_table(:,7) - Frequency_table(:,3); % implied # of 'no'
    Frequency_table(:,5) = Frequency_table(:,3) ./ Frequency_table(:,7); % implied % of 'yes'
    Frequency_table(:,6) = Frequency_table(:,4) ./ Frequency_table(:,7); % implied % of 'no'

    SEA = sqrt(Frequency_table(:,5).*Frequency_table(:,6)./Frequency_table(:,3)); % s.e. for weighted data (based on Haab McConnell)

    figure(j)

    xx = linspace(0,max(bids),500);

    best_dist = Results_summary.(CTR{j}){3,1};
    best_spike = Results_summary.(CTR{j}){3,3};

    if best_spike == 1

        Betas_param = Results1.(CTR{j}).(best_dist).beta';
        pSpike = normcdf(Betas_param(end)); % p1
        wblF = pSpike + (1-pSpike).*cdf((best_dist),xx,Betas_param(1),Betas_param(2));
        Betas_param_sim = mvnrnd(Betas_param,Results1.(CTR{j}).(best_dist).ihess,sim1)';
        wblF_sim = zeros(sim1,500);
        parfor i = 1:sim1
            pSpike = normcdf(Betas_param_sim(end,i)); % p1
            wblF_sim(i,:) = pSpike + (1-pSpike).*cdf((best_dist),xx,Betas_param_sim(1,i),Betas_param_sim(2,i));
        end

    else

        Betas_param = Results0.(CTR{j}).(best_dist).beta';
        pSpike = 0;
        wblF = pSpike + (1-pSpike).*cdf((best_dist),xx,Betas_param(1),Betas_param(2));
        Betas_param_sim = mvnrnd(Betas_param,Results0.(CTR{j}).(best_dist).ihess,sim1)';
        wblF_sim = zeros(sim1,500);
        parfor i = 1:sim1
            wblF_sim(i,:) = cdf((best_dist),xx,Betas_param_sim(1,i),Betas_param_sim(2,i));
        end

    end

    Betas_param_n = Results0.(CTR{j}).Normal.beta';
    wblF_n = cdf('Normal',xx,Betas_param_n(1),Betas_param_n(2));
    Betas_param_n_sim = mvnrnd(Betas_param_n,Results0.(CTR{j}).Normal.ihess,sim1)';
    wblF_n_sim = zeros(sim1,500);
    parfor i = 1:sim1
        wblF_n_sim(i,:) = cdf('Normal',xx,Betas_param_n_sim(1,i),Betas_param_n_sim(2,i));
    end


    wblF_sim_l = quantile(wblF_sim,0.025);
    wblF_sim_u = quantile(wblF_sim,0.975);
    wblF_sim_n_l = quantile(wblF_n_sim,0.025);
    wblF_sim_n_u = quantile(wblF_n_sim,0.975);
    hold on;
    KMplot(1) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)],'b-','color','#9999ff'); % Taking NO answers
    KMplot(2) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)+norminv(0.025)*[0;SEA(2:end)]],'b--','color','#9999ff');
    KMplot(3) = stairs([Frequency_table(:,1)],[Frequency_table(:,6)+norminv(0.975)*[0;SEA(2:end)]],'b--','color','#9999ff');
    DISTplot(1) = plot(xx,wblF,'r-','color','#006622');
    DISTplot(2) = plot(xx,wblF_sim_l,'r--','color','#006622');
    DISTplot(3) = plot(xx,wblF_sim_u,'r--','color','#006622');

    DISTplot_n(1) = plot(xx,wblF_n,'r-','color','#A2142F');
    DISTplot_n(2) = plot(xx,wblF_sim_n_l,'r--','color','#A2142F');
    DISTplot_n(3) = plot(xx,wblF_sim_n_u,'r--','color','#A2142F');

    set(KMplot(1),'linewidth',2);
    set(DISTplot(1),'linewidth',2);
    set(DISTplot_n(1),'linewidth',2);
    % hold off
    xlabel('WTP'); ylabel('Probability'); title(['Empirical CDF - ',CTR{j}])
    % xlabel('WTP'); ylabel('Probability'); title(['Empirical CDF - Poland (CAPI)'])
    % xlabel('WTP'); ylabel('Probability'); title(['Empirical CDF - Poland (CAWI)'])


    set(gca, 'YGrid', 'on', 'GridLineStyle', ':')
    ylim([0,1])
    % set(gca,'XTick', freq_tableA(:,1))
    % set(gca,'XTick', Frequency_table(:,1))
    set(gca,'Position',[.13 .11 .85 .82]);
    set(gcf,'Color', 'w')

    legend([KMplot(1), DISTplot(1),DISTplot_n(1)],'Lewbel-Watanabe (lower bound)',['Fitted ' best_dist ' distribution' best_spike.*' (with zero inflation)'],'Fitted normal distribution', 'Location', 'southeast')

    saveas(gcf,['ctr',num2str(j)],'png')

end
