clear all; clc; %#ok<CLALL>
DATA = load('BP_DATA.mat');

% response variable (based on 2'nd vote (only asked if q24 &q25 = 1 (respondents who said yes given the chance to change their mind))
DATA.q45(isnan(DATA.q45)) = 0;
DATA.q45(DATA.q45==2) = 0; % 2 = respondent changed his vote to no when given the chance

% respondents income:
DATA.income = DATA.q21;
DATA.inc_range = (DATA.q22==1).*10000 + (DATA.q22==2).*12500 + (DATA.q22==3).*20000 + (DATA.q22==4).*30000 + ...
    (DATA.q22==5).*42500 + (DATA.q22==6).*62500 + (DATA.q22==7).*87500 + (DATA.q22==8).*125000 + ...
    (DATA.q22==9).*175000 + (DATA.q22==10).*200000;
DATA.income(isnan(DATA.income)) = DATA.inc_range(isnan(DATA.income));
% dstats(DATA.income,1)
DATA.income(DATA.income==0) = 200000; % arbitrary imputation of max income bracket (used as upper bound of WTP)

% bounds of individual WTP
DATA.bounds = [DATA.bidvalue.*(DATA.q45==1),DATA.bidvalue.*(DATA.q45==0)];
DATA.bounds(DATA.bounds(:,1)==0,1) = 0;
% INPUT.bounds(INPUT.bounds(:,2)==0,2) = Inf; %this can be used for the analysis of sensitivity wrt upper bound
% DATA.bounds(DATA.bounds(:,2)==0,2) = 1000;
DATA.bounds(DATA.bounds(:,2)==0,2) = DATA.income(DATA.bounds(:,2)==0);


%% parametric analysis:

try

INPUT.MaxIter = 1e3;
INPUT.HessEstFix = 3; % available options: 0 - retained from optimization, 1 - BHHH based, 2 - numerical high-precision Jacobian based, 3 - numerical Hessian based
INPUT.SimStats = 1; % simulate descriptive statistics of the distribution
INPUT.Censor0 = 1; % this treats WTP < 0 as 0

R0 = load('results B.mat');

% version A:
DATA.filter = DATA.version == 0; % version = [0,1] - damage description (scope) A (smaller) and B (larger)
INPUT.bounds = DATA.bounds(DATA.filter,:);
INPUT.bounds = INPUT.bounds/100; % rescale - results in 100 USD
  
INPUT.Spike = 0; % no zero inflation
INPUT.Results = R0.Results_A0;
% INPUT.Results = [];
[Results_A0, Results_summary_A0] = FitParametricDistributions_BP(INPUT);

INPUT.Spike = 1; % with zero inflation
INPUT.Results = R0.Results_A1;
% INPUT.Results = [];
[Results_A1, Results_summary_A1] = FitParametricDistributions_BP(INPUT);

Results_summary_A0 = [Results_summary_A0(:,1:2),num2cell(zeros(size(Results_summary_A0(:,1)))),Results_summary_A0(:,3:end)];
Results_summary_A1 = [Results_summary_A1(:,1:2),num2cell(ones(size(Results_summary_A1(:,1)))),Results_summary_A1(:,3:end)];
Results_summary_A = [Results_summary_A0; Results_summary_A1];
Results_summary_A = [{'Distribution','Log-likelihood','Zero inflation','AIC/n','BIC/n','n - observations','k - parameters','WTP (mean)','WTP (s.e.)','RealMin'};Results_summary_A];
   
% version B:
DATA.filter = DATA.version == 1; % version = [0,1] - damage description (scope) A (smaller) and B (larger)
INPUT.bounds = DATA.bounds(DATA.filter,:);
INPUT.bounds = INPUT.bounds/100; % rescale - results in 100 USD

% INPUT.bounds(any(isnan(INPUT.bounds),2),:) = [];
   
INPUT.Spike = 0; % no zero inflation
INPUT.Results = R0.Results_B0;
% INPUT.Results = [];
[Results_B0, Results_summary_B0] = FitParametricDistributions_BP(INPUT);

INPUT.Spike = 1; % with zero inflation
INPUT.Results = R0.Results_B1;
% INPUT.Results = [];
[Results_B1, Results_summary_B1] = FitParametricDistributions_BP(INPUT);

Results_summary_B0 = [Results_summary_B0(:,1:2),num2cell(zeros(size(Results_summary_B0(:,1)))),Results_summary_B0(:,3:end)];
Results_summary_B1 = [Results_summary_B1(:,1:2),num2cell(ones(size(Results_summary_B1(:,1)))),Results_summary_B1(:,3:end)];
Results_summary_B = [Results_summary_B0; Results_summary_B1];
Results_summary_B = [{'Distribution','Log-likelihood','Zero inflation','AIC/n','BIC/n','n - observations','k - parameters','WTP (mean)','WTP (s.e.)','RealMin'};Results_summary_B];
    
% save('results B')

sendmail('mik@czaj.org','MATLAB: D5 - done!');
    
catch theErrorInfo
    
sendmail('mik@czaj.org','MATLAB: D5 - error!');
rethrow(theErrorInfo)

end

load('results B.mat');

Results_summary_A(2:end,8:9) = cellfun(@(x) x*100,Results_summary_A(2:end,8:9),'un',0); %rescale back to USD
Results_summary_B(2:end,8:9) = cellfun(@(x) x*100,Results_summary_B(2:end,8:9),'un',0); %rescale back to USD


%% retain the best starting values from two sets of results:

% R1 = load('results B.mat');
% R2 = load('results ip.mat');
% 
% k = 0;
% for i=1:length(R1.Results_B0.Distributions)
%     if ((R1.Results_B0.(R1.Results_B0.Distributions{i,2}).fval < R2.Results_B0.(R2.Results_B0.Distributions{i,2}).fval) && (R2.Results_B0.(R2.Results_B0.Distributions{i,2}).RealMin == 0)) || ...
%             ((R2.Results_B0.(R2.Results_B0.Distributions{i,2}).RealMin == 0) && (R1.Results_B0.(R1.Results_B0.Distributions{i,2}).RealMin == 1))
%         k = k+1;
%         R1.Results_B0.(R1.Results_B0.Distributions{i,2}).beta = R2.Results_B0.(R2.Results_B0.Distributions{i,2}).beta;
%         R1.Results_B0.(R1.Results_B0.Distributions{i,2}).fval = R2.Results_B0.(R2.Results_B0.Distributions{i,2}).fval;
%         R1.Results_B0.(R1.Results_B0.Distributions{i,2}).RealMin = R2.Results_B0.(R2.Results_B0.Distributions{i,2}).RealMin;
%     end
%     if ((R1.Results_B1.(R1.Results_B1.Distributions{i,2}).fval < R2.Results_B1.(R2.Results_B1.Distributions{i,2}).fval) && (R2.Results_B1.(R2.Results_B1.Distributions{i,2}).RealMin == 0)) || ...
%             ((R2.Results_B1.(R2.Results_B1.Distributions{i,2}).RealMin == 0) && (R1.Results_B1.(R1.Results_B1.Distributions{i,2}).RealMin == 1))
%         k = k+1;
%         R1.Results_B1.(R1.Results_B1.Distributions{i,2}).beta = R2.Results_B1.(R2.Results_B1.Distributions{i,2}).beta;
%         R1.Results_B1.(R1.Results_B1.Distributions{i,2}).fval = R2.Results_B1.(R2.Results_B1.Distributions{i,2}).fval;
%         R1.Results_B1.(R1.Results_B1.Distributions{i,2}).RealMin = R2.Results_B1.(R2.Results_B1.Distributions{i,2}).RealMin;
%     end
% end
% k
% 
% save('results B.mat','-struct','R1') % this is the newest results file


%% non-parametric analysis
     
bids = unique(DATA.bidvalue);

% bid  #no  #yes  #total
Frequency_tableA = [bids, ...
    histcounts(DATA.bidvalue(DATA.version == 0).*(DATA.q45(DATA.version == 0)==0),[bids;Inf])', ...
    histcounts(DATA.bidvalue(DATA.version == 0).*(DATA.q45(DATA.version == 0)==1),[bids;Inf])', ...
    histcounts(DATA.bidvalue(DATA.version == 0),[bids;Inf])']; % liczebnoci zgadzaj si z tym co zaraportowano w TM10
Frequency_tableA(:,5) = Frequency_tableA(:,2)./Frequency_tableA(:,4); % % no
Frequency_tableA(:,6) = Frequency_tableA(:,3)./Frequency_tableA(:,4); % % yes

% KM (lower bound) estimate of WTP for A:
WTPA = sum((Frequency_tableA(:,1)-[0;Frequency_tableA(1:end-1,1)]).*Frequency_tableA(:,3)./Frequency_tableA(:,4)); % reported WTP = 132.36

SDA = sqrt(sum(Frequency_tableA(:,5).*(1-Frequency_tableA(:,5))./Frequency_tableA(:,4).*((Frequency_tableA(:,1)-[0;Frequency_tableA(1:end-1,1)])).^2));
%. s.e. (based on Haab McConnell is somewhat differnt than the results presented in TM10. why?

SEA = sqrt(Frequency_tableA(:,5).*Frequency_tableA(:,6)./Frequency_tableA(:,3)); % s.e. for weighted data (based on Haab McConnell)


Results_summary_A = [Results_summary_A(1,:); ...
        {'Lewbel-Watanabe','','','','','','',WTPA,SDA,''};...
        Results_summary_A(2:end,:)];
    
Frequency_tableB = [bids, ...
    histcounts(DATA.bidvalue(DATA.version == 1).*(DATA.q45(DATA.version == 1)==0),[bids;Inf])', ...
    histcounts(DATA.bidvalue(DATA.version == 1).*(DATA.q45(DATA.version == 1)==1),[bids;Inf])', ...
    histcounts(DATA.bidvalue(DATA.version == 1),[bids;Inf])']; % liczebnoci zgadzaj si z tym co zaraportowano w TM10
Frequency_tableB(:,5) = Frequency_tableB(:,2)./Frequency_tableB(:,4); % % no
Frequency_tableB(:,6) = Frequency_tableB(:,3)./Frequency_tableB(:,4); % % yes

% KM (lower bound) estimate of WTP for A:
WTPB = sum((Frequency_tableB(:,1)-[0;Frequency_tableB(1:end-1,1)]).*Frequency_tableB(:,3)./Frequency_tableB(:,4)); % reported WTP = 132.36

SDB = sqrt(sum(Frequency_tableB(:,5).*(1-Frequency_tableB(:,5))./Frequency_tableB(:,4).*((Frequency_tableB(:,1)-[0;Frequency_tableB(1:end-1,1)])).^2));
%. s.e. (based on Haab McConnell is somewhat differnt than the results presented in TM10. why?

SEB = sqrt(Frequency_tableB(:,5).*Frequency_tableB(:,6)./Frequency_tableB(:,3)); % s.e. for weighted data (based on Haab McConnell)


Results_summary_B = [Results_summary_B(1,:); ...
        {'Lewbel-Watanabe','','','','','','',WTPB,SDB,''};...
        Results_summary_B(2:end,:)];
   
   
%% extract parameters and standard errors for each model (to see if they converged to reasonable values)


Results_summary_A(1,[11,12,13,14,15]) = {'beta'};
Results_summary_A(1,[16,17,18,19,20]) = {'std'};
for i = 1:length(Results_summary_A)-2
    idx = Results_summary_A{i+2,3}; % spike indicator
    if idx == 0
        beta = Results_A0.(Results_summary_A{i+2,1}).beta';
        std = Results_A0.(Results_summary_A{i+2,1}).std';
    elseif idx == 1
        beta = Results_A1.(Results_summary_A{i+2,1}).beta';
        std = Results_A1.(Results_summary_A{i+2,1}).std';
    end
    Results_summary_A(i+2,11:11+size(beta,2)-1) = num2cell(beta);
    Results_summary_A(i+2,16:16+size(std,2)-1) = num2cell(std);
end

Results_summary_B(1,[11,12,13,14,15]) = {'beta'};
Results_summary_B(1,[16,17,18,19,20]) = {'std'};
for i = 1:length(Results_summary_B)-2
    idx = Results_summary_B{i+2,3}; % spike indicator
    if idx == 0
        beta = Results_B0.(Results_summary_B{i+2,1}).beta';
        std = Results_B0.(Results_summary_B{i+2,1}).std';
    elseif idx == 1
        beta = Results_B1.(Results_summary_B{i+2,1}).beta';
        std = Results_B1.(Results_summary_B{i+2,1}).std';
    end
    Results_summary_B(i+2,11:11+size(beta,2)-1) = num2cell(beta);
    Results_summary_B(i+2,16:16+size(std,2)-1) = num2cell(std);
end


%% drop models that did not converge


clear idx
for i = 3:size(Results_summary_A,1)
    idx(i,1) = (Results_summary_A{i,2} >= 0) | isnan(Results_summary_A{i,2}) | isinf(Results_summary_A{i,2}) | ... % criteria for LL
        any(isnan([Results_summary_A{i,8:9}]),2) | any(isinf([Results_summary_A{i,8:9}]),2) | any([Results_summary_A{i,8:9}] > 1e3,2) | any([Results_summary_A{i,8}] < 1,2) | ... % conditions on WTP and its s.e.
        any(([Results_summary_A{i,9}]) > 0.5*([Results_summary_A{i,8}]),2) | ... % s.e. > 0.5 mean WTP
        any(isnan([Results_summary_A{i,10:19}]),2) | any(isinf([Results_summary_A{i,10:19}]),2) | any([Results_summary_A{i,10:19}] > 1e3,2);% | ... % conditions on betas and stds
        %any([Results_summary_A{i,10}] == 1,2); % criteria for RealMin
end
Results_summary_A(idx==1,:) = [];

clear i idx
for i = 3:size(Results_summary_B,1)
    idx(i,1) = (Results_summary_B{i,2} >= 0) | isnan(Results_summary_B{i,2}) | isinf(Results_summary_B{i,2}) | ... % criteria for LL
        any(isnan([Results_summary_B{i,8:9}]),2) | any(isinf([Results_summary_B{i,8:9}]),2) | any([Results_summary_B{i,8:9}] > 1e3,2) | any([Results_summary_B{i,8}] < 1,2) | ... % conditions on WTP and its s.e.
        any(([Results_summary_B{i,9}]) > 0.5*([Results_summary_B{i,8}]),2) | ... % s.e. > 0.5 mean WTP
        any(isnan([Results_summary_B{i,10:19}]),2) | any(isinf([Results_summary_B{i,10:19}]),2) | any([Results_summary_B{i,10:19}] > 1e3,2);% | ... % conditions on betas and stds
        %any([Results_summary_B{i,10}] == 1,2); % criteria for RealMin
end
Results_summary_B(idx==1,:) = [];

clear i idx


%% sort results according to fit

Results_summary_A(3:end,:) = sortrows(Results_summary_A(3:end,:),[5,4]);

Results_summary_B(3:end,:) = sortrows(Results_summary_B(3:end,:),[5,4]);


figure(1)
% [empF1A,x1A,empFloA,empFupA] = ecdf(DATA.bidvalue(DATA.version == 0),'censoring',DATA.q45(DATA.version == 0));
% [empF1A,x1A,empFloA,empFupA] = ecdf(DATA.bidvalue((DATA.q45 == 1) & (DATA.version == 0)),'censoring',ones(size(DATA.bidvalue((DATA.q45 == 1) & (DATA.version == 0)))));
[WTPA,freq_tableA] = KaplanMeierEstimator(DATA.bidvalue(DATA.version == 0),DATA.q45(DATA.version == 0));
xx = linspace(1,435,500);
[wblF,wblFlo,wblFup] = normcdf(xx,162.905081773729,137.020748787282,[16.0534086126632,3.66742329978942;3.66742329978942,9.51260311358461]);
Betas_param = Results_A0.BirnbaumSaunders.beta';
wblFB = cdf('BirnbaumSaunders',xx/100,Betas_param(1),Betas_param(2));

% Betas_param = Results_A0.Normal.beta';
% wblFB = cdf('Normal',xx,Betas_param(1),Betas_param(2));

sim1 = 1e5;
Betas_param_sim = mvnrnd(Betas_param,Results_A0.BirnbaumSaunders.ihess,sim1)';
wblFB_sim = zeros(sim1,500);
parfor i = 1:sim1
%     pSpike = normcdf(Betas_param_sim(2,i)); % p1
%     wblF_sim(i,:) = pSpike + (1-pSpike).*cdf('Exponential',xx,Betas_param_sim(1,i));    
    wblFB_sim(i,:) = cdf('BirnbaumSaunders',xx/100,Betas_param_sim(1,i),Betas_param_sim(2,i));    
end
wblFB_sim_l = quantile(wblFB_sim,0.025);
wblFB_sim_u = quantile(wblFB_sim,0.975);
figure
hold on;
KMplot(1) = stairs([0;freq_tableA(:,1)],[0;freq_tableA(:,6)],'b-');
KMplot(1) = stairs([freq_tableA(:,1)],[freq_tableA(:,6)],'b-','color','#9999ff'); % Taking NO answers
KMplot(2) = stairs([freq_tableA(:,1)],[freq_tableA(:,6)+norminv(0.025)*[0;SEA(2:end)]],'b--','color','#9999ff');
KMplot(3) = stairs([freq_tableA(:,1)],[freq_tableA(:,6)+norminv(0.975)*[0;SEA(2:end)]],'b--','color','#9999ff');

DISTplot = plot(xx,wblF,'r-',xx,wblFlo,'r:',xx,wblFup,'r:','color','#A2142F');
DISTplotB(1) = plot(xx,wblFB,'r-','color','#006622');
DISTplotB(2) = plot(xx,wblFB_sim_l,'r--','color','#006622');
DISTplotB(3) = plot(xx,wblFB_sim_u,'r--','color','#006622');

set(KMplot(1),'linewidth',2);
set(DISTplot(1),'linewidth',2);
set(DISTplotB(1),'linewidth',2);

hold off
xlabel('WTP'); ylabel('Probability'); title('Empirical CDF - Smaller set of injuries')
% set(gca,'YTick', empF1A)
set(gca,'YTick', freq_tableA(:,6))
set(gca, 'YGrid', 'on', 'GridLineStyle', ':')
ylim([0,1])
% set(gca,'XTick', x1A(2:end))
set(gca,'XTick', freq_tableA(:,1))
set(gca,'Position',[.13 .11 .85 .82]);
set(gcf,'Color', 'w')
legend([KMplot(1), DISTplotB(1),DISTplot(1)],'Lewbel-Watanabe (lower bound)','Fitted Birnbaum-Saunders distribution','Fitted normal distribution', 'Location', 'southeast')

saveas(gcf,'BPA_fit','png')


figure(2)
% [empF1A,x1A,empFloA,empFupA] = ecdf(DATA.bidvalue(DATA.version == 0),'censoring',DATA.q45(DATA.version == 0));
% [empF1A,x1A,empFloA,empFupA] = ecdf(DATA.bidvalue((DATA.q45 == 1) & (DATA.version == 0)),'censoring',ones(size(DATA.bidvalue((DATA.q45 == 1) & (DATA.version == 0)))));
[WTPA,freq_tableB] = KaplanMeierEstimator(DATA.bidvalue(DATA.version == 1),DATA.q45(DATA.version == 1));
xx = linspace(1,435,500);
[wblF,wblFlo,wblFup] = normcdf(xx/100,Results_B0.Normal.beta(1),Results_B0.Normal.beta(2),[Results_B0.Normal.ihess]);
Betas_param = Results_B0.BirnbaumSaunders.beta';
wblFB = cdf('BirnbaumSaunders',xx/100,Betas_param(1),Betas_param(2));

% Betas_param = Results_A0.Normal.beta';
% wblFB = cdf('Normal',xx,Betas_param(1),Betas_param(2));

sim1 = 1e5;
Betas_param_sim = mvnrnd(Betas_param,Results_B0.BirnbaumSaunders.ihess,sim1)';
wblFB_sim = zeros(sim1,500);
parfor i = 1:sim1
%     pSpike = normcdf(Betas_param_sim(2,i)); % p1
%     wblF_sim(i,:) = pSpike + (1-pSpike).*cdf('Exponential',xx,Betas_param_sim(1,i));    
    wblFB_sim(i,:) = cdf('BirnbaumSaunders',xx/100,Betas_param_sim(1,i),Betas_param_sim(2,i));    
end
wblFB_sim_l = quantile(wblFB_sim,0.025);
wblFB_sim_u = quantile(wblFB_sim,0.975);
figure
hold on;
KMplot(1) = stairs([0;freq_tableB(:,1)],[0;freq_tableB(:,6)],'b-');
KMplot(1) = stairs([freq_tableB(:,1)],[freq_tableB(:,6)],'b-','color','#9999ff'); % Taking NO answers
KMplot(2) = stairs([freq_tableB(:,1)],[freq_tableB(:,6)+norminv(0.025)*[0;SEA(2:end)]],'b--','color','#9999ff');
KMplot(3) = stairs([freq_tableB(:,1)],[freq_tableB(:,6)+norminv(0.975)*[0;SEA(2:end)]],'b--','color','#9999ff');

DISTplot = plot(xx,wblF,'r-',xx,wblFlo,'r:',xx,wblFup,'r:','color','#A2142F');
DISTplotB(1) = plot(xx,wblFB,'r-','color','#006622');
DISTplotB(2) = plot(xx,wblFB_sim_l,'r--','color','#006622');
DISTplotB(3) = plot(xx,wblFB_sim_u,'r--','color','#006622');

set(KMplot(1),'linewidth',2);
set(DISTplot(1),'linewidth',2);
set(DISTplotB(1),'linewidth',2);

hold off
xlabel('WTP'); ylabel('Probability'); title('Empirical CDF - Larger set of injuries')
% set(gca,'YTick', empF1A)
set(gca,'YTick', freq_tableB(:,6))
set(gca, 'YGrid', 'on', 'GridLineStyle', ':')
ylim([0,1])
% set(gca,'XTick', x1A(2:end))
set(gca,'XTick', freq_tableB(:,1))
set(gca,'Position',[.13 .11 .85 .82]);
set(gcf,'Color', 'w')
legend([KMplot(1), DISTplotB(1),DISTplot(1)],'Lewbel-Watanabe (lower bound)','Fitted Birnbaum-Saunders distribution','Fitted normal distribution', 'Location', 'southeast')

saveas(gcf,'BPB_fit','png')


% Comparison of the specification fit to the data between the normal and the best-fitting Birnbaum-Sanders distribution for the Deepwater Horizon damage assessment study (the case of the larger set of injuries)
% xlabel('WTP'); 
% ylabel('Probability of a ''no'' vote'); 
% title(["Comparison of the specification fit to the data","Normal vs. the best-fitting Birnbaum-Sanders distribution"])
% subtitle(['Deepwater Horizon damage assessment study (larger set of injuries)'])
% set(gca,'Position',[.13 .11 .85 .75]);
% legend([KMplot(1), DISTplotB(1),DISTplot(1)],'Lewbel-Watanabe (lower bound)','Fitted Birnbaum-Saunders distribution (no zero inflation)','Fitted normal distribution', 'Location', 'southeast')
% saveas(gcf,'BPB_fit_abstract','png')

